@@ -19,33 +19,83 @@ with 'Config::Model::Role::FileHandler';
 my $logger = get_logger("Backend::Dpkg::Root");
 my $user_logger = get_logger('User');
 
-my %dispatch = (
+my %hash_dispatch = (
     patches => \&read_patch_series,
     install => \&read_install_files,
+    examples => \&read_examples_files,
+    'lintian-overrides' => \&read_lintian_overrides,
 );
-
 around read_hash => sub ( $orig, $self, $obj, $elt, $file, $check, $args ) {
 
-    $logger->info("called for $elt ".$obj->location." file $file" );
-    my $method = $dispatch{$elt} // $orig;
+    $logger->info("around read_hash called for $elt ".$obj->location." file $file" );
+    my $method = $hash_dispatch{$elt} // $orig;
 
     # $file was made from element name. This does not match the actual
     # files, so we drop it
     $self->$method( $obj, $elt, $check, $args );
 };
 
+sub read_examples_files ( $self, $hash, $elt, $check, $args ) {
+    my $dir = $self->get_tuned_config_dir(%$args);
+
+    return unless $dir->exists;
+
+    $logger->info("Checking $elt directory ($dir) for ".$hash->location );
+    foreach my $file ($dir->children(qr/\.examples$/)) {
+        my $pkg =  $file->basename(qr/\.examples$/);
+        $logger->info("examples: found $pkg examples file");
+        # Just create the element. The read backend will kick in by itself
+        $hash->fetch_with_id($pkg);
+    }
+    return;
+}
+
 sub read_install_files ( $self, $hash, $elt, $check, $args ) {
     my $dir = $self->get_tuned_config_dir(%$args);
 
     return unless $dir->exists;
 
     $logger->info("Checking $elt directory ($dir) for ".$hash->location );
+    if ($dir->child('install')->exists) {
+        $hash->fetch_with_id('.');
+    }
     foreach my $file ($dir->children(qr/\.install$/)) {
         my $pkg =  $file->basename(qr/\.install$/);
         $logger->info("install: found $pkg install file");
         # Just create the element. The read backend will kick in by itself
         $hash->fetch_with_id($pkg);
     }
+    foreach my $file ($dir->children(qr/\.install\./)) {
+        my ($pkg, $arch) = split /\.install\./, $file->basename();
+        $logger->info("install: found $pkg and $arch install file");
+        $hash->fetch_with_id("$pkg/$arch");
+    }
+    foreach my $file ($dir->children(qr/^install\./)) {
+        my ($arch) = ($file->basename() =~ s/^install\.//r) ;
+        $logger->info("install: found $arch install file");
+        $hash->fetch_with_id("./$arch");
+    }
+    return;
+}
+
+sub read_lintian_overrides ( $self, $hash, $elt, $check, $args ) {
+    my $dir = $self->get_tuned_config_dir(%$args);
+
+    return unless $dir->exists;
+
+    $logger->info("Checking $elt directory ($dir) for ".$hash->location );
+
+    my $plain_file = $dir->child('lintian-overrides');
+    if ($plain_file->exists) {
+        $hash->fetch_with_id('.')->store($plain_file->slurp_utf8);
+    }
+
+    foreach my $file ($dir->children(qr/\.lintian-overrides$/)) {
+        my $pkg =  $file->basename(qr/\.lintian-overrides$/);
+        $logger->info("found $pkg lintian-overrides file");
+        $hash->fetch_with_id($pkg)->store($file->slurp_utf8);
+    }
+    return;
 }
 
 sub read_patch_series ( $self, $hash, $elt, $check, $args ) {
@@ -89,12 +139,14 @@ sub read_patch_series ( $self, $hash, $elt, $check, $args ) {
             $logger->info("found patch $pname, stored in $location ($obj)");
         }
     }
+    return;
 }
 
-sub write {
-    my $self = shift;
-    my %args = @_;
+my %write_hash_dispatch = (
+    'lintian-overrides' => \&write_lintian_overrides,
+);
 
+sub write ($self, %args) { ## no critic (ProhibitBuiltinHomonyms)
     # args are:
     # object     => $obj,         # Config::Model::Node object
     # root       => './my_test',  # fake root directory, userd for tests
@@ -116,6 +168,10 @@ sub write {
         my $file = $dir->child($elt);
 
         my $obj = $args{object}->fetch_element( name => $elt );
+        if (my $method = $write_hash_dispatch{$elt}) {
+            $self->$method($dir,$obj);
+            next;
+        }
         my $type = $obj->get_type;
         my @v;
         my $skip = 0;
@@ -149,6 +205,19 @@ sub write {
     return 1;
 }
 
+sub write_lintian_overrides ($self, $dir, $hash) {
+    foreach my $name ($hash->fetch_all_indexes) {
+        my $file = $name eq '.' ? 'lintian-overrides'
+            : $name.'.lintian-overrides';
+
+        if (my $content = $hash->fetch_with_id($name)->fetch) {
+            $logger->debug( "Dpkg writing $name in $dir");
+            $dir->child($file)->spew_utf8($content);
+        }
+    }
+    return;
+}
+
 no Mouse;
 __PACKAGE__->meta->make_immutable;
 
@@ -170,17 +239,15 @@ Config::Model::Backend::Dpkg - Read and write config as plain file
 
  my $inst = $model->create_config_class(
     name => "WithDpkg",
-    element => [ 
+    element => [
         [qw/source new/] => { qw/type leaf value_type uniline/ },
     ],
-    read_config  => [ 
-        { 
-            backend => 'plain_file', 
-            config_dir => '/tmp',
-        },
-    ],
+    rw_config  => {
+            backend => 'Dpkg',
+            config_dir => 'debian',
+    },
  );
- 
+
  my $inst = $model->instance(root_class_name => 'WithDpkg' );
  my $root = $inst->config_root ;
 
@@ -188,48 +255,34 @@ Config::Model::Backend::Dpkg - Read and write config as plain file
 
  $inst->write_back ;
 
-Now C</tmp> directory will contain 2 files: C<source> and C<new> 
+Now C<debian> directory will contain 2 files: C<source> and C<new>
 with C<foo> and C<yes> inside.
 
 =head1 DESCRIPTION
 
 This module is used directly by L<Config::Model> to read or write the
-content of a configuration tree written in several files. 
-Each element of the node is written in a plain file.
+content of a Debian package files.  Each element of the Dpkg node is
+written in a plain file.
 
-This module supports currently only leaf and list elements.  
+This module supports currently only leaf and list elements.
 In the case of C<list> element, each line of the file is a value of the list.
 
+This class is based on  L<Config::Model::Backend::PlainFile> and overrides reading of
+files in C<debian/patches> and C<debian/*install> files.
 
-=head1 CONSTRUCTOR
+=head1 Methods
 
-=head2 new
-
-Parameters: C<< node => $node_obj, name => 'plain_file' >>
-
-Inherited from L<Config::Model::Backend::Any>. The constructor will be
-called by L<Config::Model::AutoRead>.
-
-=head2 read
-
-Read data from Dpkg files
-
-When a file is read, C<read()> returns 1.
-
-=head2 write
-
-Write Dpkg data to files.
-
-C<write()> returns 1.
+See L<Config::Model::Backend::PlainFile>.
 
 =head1 AUTHOR
 
-Dominique Dumont, (ddumont at cpan dot org)
+Dominique Dumont, (dod at debian dot org)
 
 =head1 SEE ALSO
 
-L<Config::Model>, 
+L<Config::Model>,
 L<Config::Model::BackendMgr>,
-L<Config::Model::Backend::Any>, 
+L<Config::Model::Backend::Any>,
+L<Config::Model::Backend::PlainFile>,
 
 =cut
