@@ -3,7 +3,7 @@ package Config::Model::Dpkg::Copyright ;
 use strict;
 use warnings;
 
-use 5.20.0;
+use 5.020;
 use IO::Pipe;
 
 use feature qw/postderef signatures/;
@@ -14,7 +14,7 @@ use Path::Tiny;
 use Data::Dumper;
 
 use Config::Model::DumpAsData;
-use Dpkg::Copyright::Scanner qw/scan_files __squash __pack_files __pack_copyright
+use Dpkg::Copyright::Scanner qw/scan_files __squash_tree_of_copyright_ids __pack_files __pack_copyright
                                 __create_tree_leaf_from_paths __from_copyright_structure/;
 use Scalar::Util qw/weaken/;
 use Storable qw/dclone/;
@@ -26,8 +26,9 @@ sub get_joined_path ($self, $paths) {
 }
 
 sub split_path ($self,$path) {
-    return  sort ( ref $path ? @$path : split ( /[\s\n]+/ , $path ) );
+    return ( sort ( ref $path ? @$path : split ( /[\s\n]+/ , $path ) ) );
 }
+
 sub normalize_path ($self,$path) {
     my @paths = $self->split_path($path);
     return $self->get_joined_path(\@paths);
@@ -37,6 +38,7 @@ my $dumper = Config::Model::DumpAsData->new;
 
 sub _say ($self,$msg) {
     say $msg unless $self->{quiet};
+    return;
 }
 
 sub _get_old_data ($old_split_files, $old_split_dirs, $path) {
@@ -115,7 +117,7 @@ sub update ($self, %args) {
 
         # skip when no info is found in original data
         my $d_key;
-        if ( $new_data->{Copyright} =~ /no-info-found|unknown/i
+        if ( $new_data->{Copyright} =~ /no-info-found|unknown/xi
             and $new_data->{License}{short_name} =~ /unknown/i) {
             $new_copyrights_by_id[0] //= $new_data;
             $d_key = 0;
@@ -124,8 +126,8 @@ sub update ($self, %args) {
             # create an inventory of different file copyright and license data
             # this works like $copyrights_by_id but takes into account data coming
             # from old copyright file like comments
-            my $dumper = Data::Dumper->new([$new_data])->Sortkeys(1)->Indent(0);
-            my $datum_dump = $dumper->Dump;
+            my $data_dumper = Data::Dumper->new([$new_data])->Sortkeys(1)->Indent(0);
+            my $datum_dump = $data_dumper->Dump;
             $d_key = $data_keys{$datum_dump};
 
             if (not defined $d_key) {
@@ -168,7 +170,7 @@ sub update ($self, %args) {
 
 
     # implode files entries with same data index
-    __squash(\%new_split_files, \@new_copyrights_by_id) ;
+    __squash_tree_of_copyright_ids(\%new_split_files, \@new_copyrights_by_id) ;
 
     # pack files by copyright id
     my @packed = __pack_files(\%new_split_files);
@@ -204,12 +206,12 @@ sub update ($self, %args) {
 
             my $using_old_data = 0;
             if ($old_data and $old_data->{Copyright} and $old_data->{License}{short_name}) {
-                if ($datum->{Copyright} =~ /unknown|no-info-found/i) {
+                if ($datum->{Copyright} =~ /unknown|no-info-found/xi) {
                     $self->_say( "keeping copyright dir data for $p");
                     $datum->{Copyright} = $old_data->{Copyright};
                     $using_old_data = 1;
                 }
-                if ($datum->{License}{short_name} =~ /unknown|no-info-found/i) {
+                if ($datum->{License}{short_name} =~ /unknown|no-info-found/xi) {
                     $self->_say( "keeping license dir data for $p");
                     $datum->{License}{short_name} = $old_data->{License}{short_name};
                     $datum->{License}{full_license} = $old_data->{License}{full_license};
@@ -240,7 +242,7 @@ sub update ($self, %args) {
         # if full_license is not provided in datum, check global license(s)
         if (not $datum->{License}{full_license}) {
             my $ok = 0;
-            my @sub_licenses = split m![,\s]+(?:and/or|or|and)[,\s]+!,$l;
+            my @sub_licenses = split m![,\s]+ (?:and/or|or|and) [,\s]+!x,$l;
             my $lic_count = 0;
             my @empty_licenses = grep {
                 my $text = $self->grab_value(steps => qq!License:"$_" text!, check =>'no') ;
@@ -253,7 +255,9 @@ sub update ($self, %args) {
                 my $filler = "Please fill license $l from header of @paths";
                 if ($lic_count > 1 ) {
                     $self->_say( "Adding dummy global license text for license $l for path @paths");
-                    map { $self->load(qq!License:"$_" text="$filler"!) } @empty_licenses ;
+                    for my $lic (@empty_licenses) {
+                        $self->load(qq!License:"$lic" text="$filler"!)
+                    };
 
                 }
                 else {
@@ -265,16 +269,18 @@ sub update ($self, %args) {
         }
 
         eval {
-            $files_obj->fetch_with_id($path_str)->load_data( data => $datum, check =>'yes' );
-        };
-        if ($@) {
+            $files_obj
+                ->fetch_with_id($path_str)
+                ->load_data( data => $datum, check =>'yes' );
+            1;
+        } or do {
             die "Error: Data extracted from source file is corrupted:\n$@"
                 ."This usually mean that cme or licensecheck (or both) "
                 ."have a bug. You may work-around this issue by adding an override entry in "
                 ."fill.copyright.blanks file. See "
                 ."https://github.com/dod38fr/config-model/wiki/Updating-debian-copyright-file-with-cme "
                 ."for instructions. Last but not least, please file a bug against libconfig-model-dpkg-perl.\n";
-        }
+        };
     }
 
     # delete global license without text
@@ -326,13 +332,15 @@ sub _apply_fix_scan_copyright_file ($self, $current_dir) {
     # read a debian/fix.scanned.copyright file to patch scanned data
     my $debian = $current_dir->child('debian'); # may be missing in test environment
     if ($debian->is_dir) {
-        my @fixes = $current_dir->child('debian')->children(qr/fix\.scanned\.copyright$/);
+        my @fixes = $current_dir->child('debian')->children(qr/fix\.scanned\.copyright$/x);
         $self->_say( "Note: loading @fixes fixes from copyright fix files") if @fixes;
         foreach my $fix ( @fixes) {
-            my @l = grep { /[^\s]/ } grep { ! m!^(#|//)!  } $fix->lines_utf8;
-            eval { $self->load( steps => join(' ',@l) , caller_is_root => 1 ); };
-            my $e = $@;
-            if ($e) {
+            my @l = grep { /[^\s]/ } grep { ! m!^(?:#|//)!  } $fix->lines_utf8;
+            eval {
+                $self->load( steps => join(' ',@l) , caller_is_root => 1 );
+                1;
+            } or do {
+                my $e = $@;
                 my $msg = $e->full_message;
                 Config::Model::Exception::User->throw(
                     object => $self,
@@ -341,6 +349,7 @@ sub _apply_fix_scan_copyright_file ($self, $current_dir) {
             }
         }
     }
+    return;
 }
 
 sub _prune_old_dirs ($self, $h, $old_dirs, $path = [] ) {
@@ -359,6 +368,7 @@ sub _prune_old_dirs ($self, $h, $old_dirs, $path = [] ) {
         $self->_say( "Removing old entry $dir_path" );
         delete $old_dirs->{$dir_path};
     }
+    return;
 }
 
 1;
