@@ -36,7 +36,10 @@ my $madison_endpoint = "https://$madison_host/madison";
 
 # list of virtual packages
 # See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.yaml
-# updated from 16 May 2020 version
+# Last update: 2021-08-19
+# wget -q -O- https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.yaml | \
+# perl -MYAML -e 'BEGIN {local $/=undef; $y = <>;} $yaml=Load($y); printf (" %s\n", $_->{name}) foreach @{$yaml->{virtualPackages}};' | \
+# sort -u
 my @virtual_list = qw/
  adventure
  audio-mixer
@@ -155,34 +158,14 @@ push @virtual_list, qw/
  ssh-server
 /;
 
-# debhelper sequences
-# grep-available -F Provides dh-sequence- -s Provides -n | grep -Eo 'dh-sequence-[-a-z]+' | sort -u | xargs printf " %s\n"
-push @virtual_list, qw/
- dh-sequence-ada-library
- dh-sequence-cli
- dh-sequence-cme-upgrade
- dh-sequence-dist-zilla
- dh-sequence-dwz
- dh-sequence-elf-tools
- dh-sequence-fortran-mod
- dh-sequence-gir
- dh-sequence-gnome
- dh-sequence-installinitramfs
- dh-sequence-libva
- dh-sequence-numpy
- dh-sequence-perl-dbi
- dh-sequence-perl-imager
- dh-sequence-perl-openssl
- dh-sequence-pypy
- dh-sequence-python
- dh-sequence-scour
- dh-sequence-sphinxdoc
- dh-sequence-systemd
- dh-sequence-zathura
-/;
-
 my %virtual_hash = map {( $_ => 1); } @virtual_list;
 
+sub _is_virtual ($pkg) {
+    return 1 if $virtual_hash{$pkg};
+    return 1 if $pkg =~ /^dh-sequence-[a-z0-9-]+$/;
+    return 0;
+}
+
 my %debian_map;
 my $version = \%Module::CoreList::version;
 
@@ -356,20 +339,16 @@ sub dep_parser {
 # check_dep -> meta filter -> control maintainer -> create control class
 # autoread started -> read all fileds -> read dependency -> check_dep ...
 
-sub check_value {
-    my $self = shift ;
-    my %args = @_ > 1 ? @_ : (value => $_[0]) ;
+around check_value => sub($orig, $self, @args) {
+    my %args = @_ > 1 ? @args : (value => $args[0]) ;
 
     $args{fix} //= 0;
-	# when fixing, SUPER::check_value may modify $args{value} before calling back
-    my ($ok, $value) = $self->SUPER::check_value(%args) ;
+	# when fixing, $orig->() may modify $args{value} before calling back
+    my ($ok, $value) = $orig->($self, %args) ;
     return $self->check_dependency(%args, value => $value, ok => $ok) ;
-}
-
-sub check_dependency {
-    my $self = shift;
-    my %args = @_ ;
+};
 
+sub check_dependency ($self, %args){
     my ($value, $check, $silent, $notify_change, $ok, $apply_fix)
         = @args{qw/value check silent notify_change ok fix/} ;
 
@@ -403,6 +382,7 @@ sub check_dependency {
     foreach my $dep (@dep_chain) {
         next unless ref($dep) ; # no need to check debhelper variables
         $self->check_or_fix_pkg_name($apply_fix, $dep, $old, \@msgs) ;
+        $self->check_or_fix_too_old_pkg($apply_fix, $dep, $old, \@msgs) ;
 		$self->check_or_fix_essential_package($apply_fix, $dep, \@msgs) ;
 		$self->check_or_fix_dep($apply_fix, $dep, $old, \@msgs) ;
     }
@@ -414,13 +394,13 @@ sub check_dependency {
     my $new = $self->struct_to_dep(@dep_chain);
 
     if ( $logger->is_debug ) {
-        my $new //= '<undef>';
-        no warnings 'uninitialized';
-        $logger->debug( "'$old' done" . ( $apply_fix ? " changed to '$new'" : '' ) );
+        my $new_str = $new // '<undef>';
+        no warnings 'uninitialized'; ## no critic (ProhibitNoWarnings)
+        $logger->debug( "'$old' done" . ( $apply_fix ? " changed to '$new_str'" : '' ) );
     }
 
     {
-        no warnings 'uninitialized';
+        no warnings 'uninitialized'; ## no critic (ProhibitNoWarnings)
         my $msg = join('; ', @msgs);
         $self->_store_fix( $old, $new, $msg ) if $apply_fix and @msgs and $new ne $old;
     }
@@ -438,7 +418,7 @@ sub check_debhelper_compat_version {
     # use the compat value mentioned in debhelper-compat dependency
     my $compat_value = defined $dep_v ? ($dep_v =~ s/~$//r) : undef;
 
-    if (not defined $compat_value or $compat_value < $current_compat_value - 1 ) {
+    if (not defined $compat_value or $compat_value < $current_compat_value ) {
         if ($apply_fix) {
             $dep_info->{dep} = [ '=' , $current_compat_value ];
             my $msg = "changed debhelper-compat version dependency to $current_compat_value";
@@ -453,6 +433,7 @@ sub check_debhelper_compat_version {
             $logger->info("will warn: $msg (fix++)");
         }
     }
+    return;
 }
 
 sub check_debhelper_version {
@@ -487,6 +468,7 @@ sub check_debhelper_version {
             $logger->info("will warn: $msg (fix++)");
         }
     }
+    return;
 }
 
 sub check_compat_object ($self) {
@@ -518,6 +500,7 @@ sub check_compat_object_value ($self, $apply_fix, $msgs) {
             $logger->info("will warn: $msg (fix++)");
         }
     }
+    return;
 }
 
 sub check_compat_value ($self, $compat_value, $apply_fix, $msgs) {
@@ -540,10 +523,7 @@ sub check_compat_value ($self, $compat_value, $apply_fix, $msgs) {
     return $compat_value;
 }
 
-sub struct_to_dep {
-    my $self = shift ;
-    my @input = @_ ;
-
+sub struct_to_dep ($self, @input) {
     my @alternatives ;
     foreach my $d (@input) {
         my $line = '';
@@ -551,7 +531,7 @@ sub struct_to_dep {
         # empty name is skipped
         if (ref $d) {
             my ($name, $dep,$arch, $prof) = @{$d}{qw/name dep arch profile/} ;
-            if ( $name) {
+            if ($name) {
                 $line .= $name;
 
                 $line .= " (@$dep)" if defined $dep->[1];
@@ -677,8 +657,6 @@ sub check_versioned_dep {
 		 and defined $oper
 		 and $vers !~ /^\$/  # a dpkg variable
 	 ) {
-		my $src_pkg_name = $self->grab_value("!Dpkg::Control source Source") ;
-
 		return ($self->has_older_version_than ($pkg, $vers, \@dist_version ));
 	}
 	else {
@@ -695,8 +673,7 @@ sub has_older_version_than {
         my ($d,$v) = splice @$dist_version,0,2 ;
 
         push @list, "$d -> $v;" ;
-
-        if ($vs->compare($vers,$v) > 0 ) {
+        if ($d !~ /oldold/ and $vs->compare($vers,$v) > 0 ) {
             $has_older = 1 ;
         }
     }
@@ -737,6 +714,7 @@ sub check_or_fix_essential_package {
             $logger->info("will warn: $msg (fix++)");
         }
     }
+    return;
 }
 
 
@@ -772,7 +750,7 @@ sub check_or_fix_pkg_name {
     }
     else {
         my @res = $self->get_available_version(  $pkg );
-		if ( @res == 0 and not $virtual_hash{$pkg}) {
+		if ( @res == 0 and not _is_virtual($pkg)) {
 			# no version found for $pkg
 			# don't know how to distinguish virtual package from source package
 			$logger->debug("unknown package $pkg");
@@ -780,6 +758,43 @@ sub check_or_fix_pkg_name {
 				"package $pkg is unknown. Check for typos if not a virtual package.");
 		}
     }
+    return;
+}
+
+sub check_or_fix_too_old_pkg ( $self, $apply_fix, $dep_info, $old, $msgs ) {
+    my $pkg = $dep_info->{name};
+
+    $logger->debug("called with '", scalar $self->struct_to_dep($dep_info), "' and fix $apply_fix")
+        if $logger->is_debug;
+
+    if (not defined $pkg) {
+        # pkg may be cleaned up during fix
+        return;
+    }
+
+    my @dist_version = $self->get_available_version( $pkg ) ;
+
+    my $is_too_old = scalar @dist_version ? 1 : 0;
+    while (@dist_version) {
+        my ($d,$v) = splice @dist_version,0,2 ;
+        $is_too_old = 0 unless $d =~ /oldold/;
+    }
+
+    if ($is_too_old) {
+        $logger->debug("Found dependency on too old package: $pkg");
+        my $msg = "applied on too old package: $pkg";
+        if ($apply_fix) {
+            %$dep_info = ();
+            $logger->info("fix: removed relation with too old $pkg package");
+            push $msgs->@*, $msg;
+        }
+        else {
+            $self->add_warning($msg);
+            $self->{nb_of_fixes}++;
+            $logger->info("will warn: $msg (fix++)");
+        }
+    }
+    return $is_too_old;
 }
 
 sub check_or_fix_dep {
@@ -824,6 +839,7 @@ sub check_or_fix_dep {
         $fix_sub = sub { delete $_[0]->{dep} };
     }
     $self->warn_or_fix_dep_info ($fix_sub, $apply_fix, $dep_info, $msgs, $warn_str, $log_str) ;
+    return;
 }
 
 
@@ -840,6 +856,7 @@ sub warn_or_fix_dep_info {
         $self->add_warning( $warn_msg );
         $logger->info("will warn: $warn_msg (fix++)");
     }
+    return;
 }
 
 use vars qw/%cache $use_test_cache/ ;
@@ -872,8 +889,8 @@ sub get_available_version {
 
     $logger->debug("called on $pkg_name");
 
-    # don't query info for known virtual package
-    if ($virtual_hash{$pkg_name}) {
+    # don't query info for known or calculated virtual package
+    if (_is_virtual($pkg_name)) {
         $logger->debug("$pkg_name is a known virtual package");
         return ();
     }
@@ -930,7 +947,7 @@ sub cache_info_from_madison {
     my @needed;
 
     foreach my $pkg_name (@pkg_names) {
-        next if $virtual_hash{$pkg_name} ; # skip known virtual package
+        next if _is_virtual($pkg_name) ; # skip known or calculated virtual package
         my ($time,@res) = split / /, ($cache{$pkg_name} || '');
         if (defined $time and $time =~ /^\d+$/ and $time > $cache_expire_date) {
             $logger->debug("using cached info for $pkg_name");
@@ -963,6 +980,7 @@ sub cache_info_from_madison {
     else {
         warn "cannot get data from madison. Check your proxy ?\n";
     }
+    return;
 }
 
 # See https://ftp-master.debian.org/epydoc/dakweb.queries.madison-module.html
@@ -1065,7 +1083,7 @@ syntax as described in http://www.debian.org/doc/debian-policy/ch-relationships.
 Whether the version specified with C<< > >> or C<< >= >> is necessary.
 This module checks with Debian server whether older versions can be
 found in Debian old-stable or not. If no older version can be found, a
-warning is issued (unless the package is known to be virtual)
+warning is issued (unless the package is known or calculated to be virtual)
 
 =item *
 
@@ -1106,7 +1124,7 @@ as-is. No check are performed.
 =item *
 
 Virtual package names are found scanning local apt cache. Hence an unknown package
-on your system may a virtual package on another system.
+on your system may be a virtual package on another system.
 
 =item *
 
