@@ -51,7 +51,8 @@ include /usr/share/dpkg/default.mk
 
 # main packaging script based on dh7 syntax
 %:
-	dh $@',
+	dh $@
+',
         'description' => 'debian/rules is a makefile containing all instructions required to build a debian package.',
         'summary' => 'package build rules',
         'type' => 'leaf',
@@ -140,6 +141,21 @@ Here the mapping between the install files and the install key:
 "package/arch" -> C<debian/package.install.arch>
 
 '.'=back
+
+',
+        'index_type' => 'string',
+        'type' => 'hash'
+      },
+      'examples',
+      {
+        'cargo' => {
+          'config_class_name' => 'Dpkg::Install',
+          'type' => 'node'
+        },
+        'description' => 'List of the examples files to install into
+C</usr/share/doc/package/examples>
+
+Use the package name as the key of the hash
 ',
         'index_type' => 'string',
         'type' => 'hash'
@@ -148,7 +164,13 @@ Here the mapping between the install files and the install key:
       {
         'cargo' => {
           'type' => 'leaf',
-          'value_type' => 'uniline'
+          'value_type' => 'uniline',
+          'warn_if_match' => {
+            '^/' => {
+              'fix' => 's!^/!!',
+              'msg' => 'path should not begin with \'/\''
+            }
+          }
         },
         'description' => 'List the files that are deliberately not installed in any binary package. Paths listed in this file are (only) ignored by the check done via --list-missing (or --fail-missing). However, it is not a method to exclude files from being installed. Please use --exclude for that.
 
@@ -161,6 +183,28 @@ Please keep in mind that dh_install will not expand wildcards in this file.
         'config_class_name' => 'Dpkg::Source',
         'type' => 'node'
       },
+      'lintian-overrides',
+      {
+        'cargo' => {
+          'class' => 'Config::Model::Dpkg::Lintian::Overrides',
+          'type' => 'leaf',
+          'value_type' => 'string'
+        },
+        'description' => 'Contains the lintian overrides parameters from all lintian overrides files contained in C<debian/*lintian-overrides>.
+
+plain C<lintian-overrides> is contained in "." element.
+
+Other files are contained in basename element.
+
+For instance, C<debian/foo.lintian-overrides> is contained in C<foo> element.
+
+Unknown L<lintian tags| https://lintian.debian.org/tags.html> trigger a warning.
+
+
+',
+        'index_type' => 'string',
+        'type' => 'hash'
+      },
       'clean',
       {
         'cargo' => {
