@@ -27,7 +27,7 @@ For details, see L<section 5.7 of Debian policy|https://www.debian.org/doc/debia
       {
         'compute' => {
           'allow_override' => '1',
-          'formula' => '$ENV{PWD} =~ m!/([^/]+)$!; $1;',
+          'formula' => 'use Cwd; getcwd =~ m!/([^/]+)$!; $1;',
           'use_eval' => '1'
         },
         'description' => 'Source package name. Defaults to the name of the current directory.',
@@ -37,6 +37,14 @@ For details, see L<section 5.7 of Debian policy|https://www.debian.org/doc/debia
         'type' => 'leaf',
         'value_type' => 'uniline'
       },
+      'debhelper-version',
+      {
+        'default' => '0',
+        'description' => 'Debhelper version. This parameter is hidden because it does not exist in control. It\'s used to drive warp mechanism for parameters that  depend on debhelper version.',
+        'level' => 'hidden',
+        'type' => 'leaf',
+        'value_type' => 'integer'
+      },
       'Maintainer',
       {
         'compute' => {
@@ -257,6 +265,22 @@ in Debian developer reference.',
               'fix' => '$_ = \'libpng-dev\';',
               'msg' => 'This dependency is deprecated and should be replaced with libpng-dev. See BTS 650601 for details'
             }
+          },
+          'warp' => {
+            'follow' => {
+              'dhv' => '- debhelper-version'
+            },
+            'rules' => [
+              'defined $dhv and $dhv and $dhv+0 >= 10',
+              {
+                'warn_if_match' => {
+                  '^(autotools-dev|dh-autoreconf)' => {
+                    'fix' => '$_ = undef',
+                    'msg' => 'dependency "$_" is not necessary with debhelper > 10'
+                  }
+                }
+              }
+            ]
           }
         },
         'class' => 'Config::Model::Dpkg::DependencyList',
@@ -294,7 +318,7 @@ See also L<deb-src-control|https://manpages.debian.org/unstable/dpkg-dev/deb-src
           'value_type' => 'uniline'
         },
         'class' => 'Config::Model::Dpkg::DependencyList',
-        'description' => 'Same as Build-Depends, but these pakcages are only needed when building the architecture dependent packages. The Build-Depends are also installed in this case. 
+        'description' => 'Same as Build-Depends, but these packages are only needed when building the architecture dependent packages. The Build-Depends are also installed in this case. 
 
 See L<deb-src-control man page|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> for details',
         'duplicates' => 'warn',
