@@ -1,4 +1,7 @@
-[
+use strict;
+use warnings;
+
+return [
   {
     'accept' => [
       '.*',
@@ -23,7 +26,7 @@
           'allow_override' => '1',
           'formula' => 'require Software::LicenseMoreUtils ;
 my $lic = &index( - ) ;
-my $h = { short_name => $lic, holder => \'foo\' } ;
+my $h = { short_name => $lic, holder => \'X. Ample\' } ;
 my $text;
 
 if (defined $lic and $lic) {
@@ -35,8 +38,11 @@ if (defined $lic and $lic) {
        # need to cleanup text to mimic cleanup done when copyright
        # data is read from file
        chomp($text);
-       # work-around a bug in lgpl license text summary... my bad :-/
-       $text =~ s!^\\.!!mg;
+       # cleanup tabs, not allowed by copyright spec
+       $text =~ s!\\t!    !g;
+       # cleanup empty lines (which are also cleaned up when writing file,
+       # but this messes tests up)
+       $text =~ s!\\n +\\n!\\n\\n!g;
    }
 # FIXME: find a way to warn user if a license is unknown only when
 # text is not set by another mean... may loop bad if not careful
