@@ -1,4 +1,7 @@
-[
+use strict;
+use warnings;
+
+return [
   {
     'author' => [
       'Dominique Dumont'
@@ -43,6 +46,24 @@ There may also be a .git shallow file listing revisions for a shallow git clone.
         'config_class_name' => 'Dpkg::Source::Options',
         'description' => 'Source options as described in L<dpkg-source>',
         'type' => 'node'
+      },
+      'lintian-overrides',
+      {
+        'class' => 'Config::Model::Dpkg::Lintian::Overrides',
+        'description' => 'Contains the lintian overrides parameters from all lintian overrides files contained in C<debian/*lintian-overrides>.
+
+plain C<lintian-overrides> is contained in "." element.
+
+Other files are contained in basename element.
+
+For instance, C<debian/foo.lintian-overrides> is contained in C<foo> element.
+
+Unknown L<lintian tags| https://lintian.debian.org/tags.html> trigger a warning.
+
+
+',
+        'type' => 'leaf',
+        'value_type' => 'string'
       }
     ],
     'license' => 'LGPL2',
