@@ -4,19 +4,27 @@ use strict;
 use warnings;
 
 use 5.20.0;
-use IO::Pipe;
 use Exporter::Lite;
 use Array::IntSpan;
 use Path::Tiny;
+use Time::localtime;
 use Carp;
-use YAML::Tiny;
+use TOML::Tiny qw/from_toml/;
+use YAML::XS qw/LoadFile/;
+use JSON;
+
+$YAML::XS::LoadBlessed = 0;
 
 use feature qw/postderef signatures/;
 no warnings qw/experimental::postderef experimental::signatures/;
 
-binmode STDOUT, ':utf8';
+binmode STDOUT, ':encoding(utf8)';
 
-our @EXPORT = qw(scan_files print_copyright __from_copyright_structure __to_copyright_structure);
+our @EXPORT_OK = qw(scan_files print_copyright
+                    __create_tree_leaf_from_paths
+                    __from_copyright_structure __pack_files
+                    __pack_copyright __squash_tree_of_copyright_ids
+                    __to_copyright_structure);
 
 my $whitespace_list_delimiter = $ENV{'whitespace_list_delimiter'} || "\n ";
 
@@ -25,7 +33,7 @@ my $whitespace_list_delimiter = $ENV{'whitespace_list_delimiter'} || "\n ";
 # derivative work to LGPL-2.1+ was given by Jonas.
 # see https://lists.alioth.debian.org/pipermail/pkg-perl-maintainers/2015-March/084900.html
 
-# Copyright 2014 Dominique Dumont <dod@debian.org>
+# Copyright 2014-2020 Dominique Dumont <dod@debian.org>
 # Copyright © 2005-2012 Jonas Smedegaard <dr@jones.dk>
 # Description: Reformat licencecheck output to copyright file format
 #
@@ -52,7 +60,9 @@ sub print_copyright ( %args ) {
     }
 
     # regroup %files hash: all leaves have same id -> wild card
-    __squash($split_files, $copyrights_by_id);
+    if (!exists $args{long} || $args{long} != 1) {
+        __squash_tree_of_copyright_ids($split_files, $copyrights_by_id);
+    }
 
     # pack files by copyright id
     my @packed = __pack_files($split_files);
@@ -79,15 +89,17 @@ sub print_copyright ( %args ) {
         $args{out}->spew_utf8( @out);
     }
     else {
-        binmode(STDOUT, ":utf8");
+        binmode(STDOUT, ":encoding(UTF-8)");
         print @out;
     }
+    return;
 }
 
 my $quiet;
 
 sub _warn ($msg) {
     warn $msg unless $quiet;
+    return;
 }
 
 my %default ;
@@ -96,24 +108,27 @@ $default{ignore}= << 'EOR';
 # Ignore general backup files
 ~$|
 # Ignore emacs recovery files
-(?:^|/)\.#|
+(^|/)\.#|
 # Ignore vi swap files
-(?:^|/)\..*\.swp$|
+\.swp$|
 # Ignore baz-style junk files or directories
-(?:^|/),,.*(?:$|/.*$)|
+(^|/),,.*(?:$|/.*$)|
 # File-names that should be ignored (never directories)
-(?:^|/)(?:DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
+(^|/)(DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
 # File or directory names that should be ignored
-(?:^|/)(?:CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
+(^|/)(CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
 \.shelf|_MTN|\.bzr(?:\.backup|tags)?)(?:$|/.*$)|
 # skip debian files that are too confusing or too short
-(?:^|/)debian/((fix.scanned.)?copyright|changelog|NEWS|compat|.*yml|docs|source|patches/series)
+(?:^|/)debian/((fix.scanned.)?copyright|changelog|NEWS|compat|.*yml|docs|source|patches/series)|
+# skip some binary files
+(png|jpg|pdf|ico|bmp|jpe?g)$
 EOR
 
 # also from licensecheck
 $default{check} = << 'EOR2' ;
     (^|/)\w+$                 # scripts, AUTHORS files
     |^README                   # README.* files
+    |META\d?.json                # Perl or Raku meta file
     |\.(                          # search for file suffix
         c(c|pp|xx)?              # c and c++
        |h(h|pp|xx)?              # header files for c and c++
@@ -132,9 +147,11 @@ $default{check} = << 'EOR2' ;
        |sh
        |php
        |py(|x)
+       |rs                       # rust
        |rb
+       |rs
        |java
-       |js
+       |js(on)?
        |vala
        |el
        |sc(i|e)
@@ -145,6 +162,7 @@ $default{check} = << 'EOR2' ;
        |mod
        |m
        |md|markdown
+       |toml
        |tex
        |mli?
        |(c|l)?hs
@@ -153,23 +171,15 @@ $default{check} = << 'EOR2' ;
 EOR2
 
 # cleanup the regexp
-map { s/#.*\n//g;  s/[\s\n]+//g; } values %default;
-
-# option to skip UNKNOWN ?
-# load a file to override some entries ?
-sub scan_files ( %args ) {
-
-    $quiet = $args{quiet} // 0;
+for (values %default) { s/#.*\n//g;  s/[\s\n]+//g; };
 
+sub _get_data_from_files  ( %args ) {
     my $current_dir = $args{from_dir} || path('.');
 
-    my %regexps = %default; # default is needed during tests 
+    my %regexps = %default; # default is needed during tests
     my @lines ;
-    if (my $file = $ENV{COPYRIGHT_SCANNER_INPUT}) {
-        @lines = path($file)->lines_utf8 ; # for tests
-    }
-    elsif ($args{in}) {
-        @lines = $args{in}->lines_utf8; # for other tests
+    if ($args{in}) {
+        @lines = $args{in}->lines_utf8; # for tests
     }
     else {
         my $scan_data = {};
@@ -177,148 +187,51 @@ sub scan_files ( %args ) {
         my $scan_patterns = $debian->child("copyright-scan-patterns.yml");
 
         if ($debian->is_dir and $scan_patterns->is_file) {
-            my $yaml = $scan_patterns->slurp_utf8;
-            # { check => { suffixes => [ js pl ], pattern => '[A-Z]*'}, ignore => { suffixes => [ jpg png ], pattern => 'foo.*'} }
-            $scan_data = Load $yaml;
+            $scan_data = LoadFile($scan_patterns->stringify);
         }
 
         # licensecheck --check is broken ( #842368 ), so --skipped option is useless.
         # let's scan everything and skip later
         foreach my $what (qw/check ignore/) {
-            my $data = $scan_data->{$what} or next;
+            my $data = $scan_data->{$what} || {};
             my $reg = join(
                 '|' ,
                 (map { '\.'.$_.'$'} @{$data->{suffixes} || []}),
                 @{ $data->{pattern} || []},
                 $default{$what}
             );
-            $regexps{$what} = qr/$reg/;
+            $regexps{$what} = $reg;
         }
 
-        my $pipe = IO::Pipe->new();
-        my @cmd = (qw/licensecheck --encoding utf8 --copyright --machine --deb-fmt --recursive/,
-                   '--ignore='.$regexps{ignore}, '.');
-        $pipe->reader(@cmd);
+        my $lic_cmd = 'licensecheck --encoding utf8 --copyright --machine --shortname-scheme=debian,spdx --recursive '
+            . "'--ignore=%s' %s";
+        my $cmd = sprintf($lic_cmd, $regexps{ignore}, $current_dir);
+        open(my $pipe, "-|", $cmd) or die "Can't open '$cmd' : $!\n";;
         binmode($pipe, ":encoding(UTF-8)");
-        @lines = $pipe->getlines;
-        $pipe->close;
+        my $length = length($current_dir->stringify) + 1;
+        @lines = map { substr $_,$length } $pipe->getlines;
+        $pipe->close or die $! ? "Error closing licensecheck pipe: $!\n"
+                               : "Exit status ".$?." $cmd\n";
     }
+    return _trim_lines (\@lines, \%regexps);
+}
 
-    my $fill_blank_data = __load_fill_blank_data($current_dir);
-
-    my %copyrights ;
-    my $files = {};
-    my $id = 1;
-    my @no_info_list;
+sub _trim_lines ($lines, $regexps) {
+    my @kept_lines;
     my @skipped;
-
-    foreach my $line (sort @lines) {
+    foreach my $line (sort $lines->@*) {
         chomp $line;
-        # say "found: $line";
         my ($f,$l,$c) = split /\t/, $line;
         $f =~ s!\./!!;
-        next if $files->{$f}; # file already parsed
-
-        # skip copyright because there's no need to have recursive copyrights
-        # skip changelog because it's often contains log entries beginning with copyright
-        # and it's a collective work by nature.
-        next if $f =~  m!debian/(copyright|changelog)!;
-
-        # skip copyright infos extracted from license files when they
-        # come from the author of the license and not from the project
-        # author
-        if ($f =~  m/^COPYING|LICENSE$/) {
-            if ($c =~ /Free Software Foundation/) { next; }
-            if ($c =~ /Perl Foundation/) { next; }
-        }
-
-
-        # skip license files without copyright information (usually plain license)
-        next if $f =~  m/^COPYING|LICENSE$/ && $c =~ /No copyright/;
 
         # filter out files user does not want checked
-        if ( $f !~ $regexps{check} ) {
+        if ( $f !~ $regexps->{check} ) {
             # the ignore test should not be usefull as the files are ignored by licensecheck
-            push @skipped, $f unless $f =~ $regexps{ignore};
-            next;
+            push @skipped, $f unless $f =~ $regexps->{ignore};
         }
-
-        # $l =~ s/([*?\\])/\\$1/g;
-        # $l =~ s/\s*\(unversioned\/unknown version\)//g;
-        # $l =~ s/\s*\(with incorrect FSF address\)//g;
-        # $l =~ s/(\w+)\s+\(v([^)]+) or v([^)]+)\)/uc($1)."-$2 or ".uc($1)."-$3"/eg;
-        # $l =~ s/\s+\(v([^)]+) or later\)/-$1+/g;
-        # $l =~ s/\s+\(v([^)]+)\)/-$1/g;
-        # $l =~ s/^\s*(GENERATED FILE)/UNKNOWN/g;
-        # $l =~ s/\s+(GENERATED FILE)//g;
-        # $l =~ s/\bzlib\/libpng\b/Zlib/g;
-        # $l =~ s/\bMIT\/X11 \(BSD like\)/Expat/g;
-        # $l =~ s/\bBSD \((\d) clause\)/BSD-$1-clause/g;
-        # $l =~ s/\bpublic domain\b/public-domain/ig;
-
-        # licensecheck uses and/or when a file contains 2
-        # licenses. and/or is not a valid syntax for debian/copyright
-        # file
-        $l =~ s!and/or!or!g;
-
-        # # deduplicate license keywords and add 'or' keyword where missing
-        # # this is fragile. may need to change license-check to output license keyword
-        # my %tmp_l = map { ( $_ => 1) ;} grep { $_ ne 'or' } split(/ /,$l);
-        # $l = join(' or ', sort keys %tmp_l) ;
-
-        # Handle license like "under the same term as Perl"
-        # $l =~ s/perl/Artistic or GPL-1+/i;
-
-        $c =~ s/'//g;
-        $c =~ s/^&copy;\s*//g;
-        $c =~ s/\(c\)\s*//g;
-        $c =~ s/(?<=\b\d{4})\s*-\s*\d{4}(?=\s*-\s*(\d{4})\b)//g;
-        $c =~ s/(\d+)\s*-\s*(\d+)/$1-$2/g;
-        $c =~ s/\b(\d{4}),?\s+([\S^\d])/$1, $2/g;
-        $c =~ s/\s+by\s+//g;
-        $c =~ s/(\\n)*all\s+rights?\s+reserved\.?(\\n)*\s*//gi; # yes there are literal \n
-        $c = 'no-info-found' if $c =~ /^\*No/;
-        $c =~ s/\(r\)//g;
-        $c =~ s!^[\s,/*]|[\s,#/*-]+$!!g;
-        $c =~ s/--/-/g;
-        $c =~ s!\s+\*/\s+! !;
-
-        $c = __pack_copyright($c);
-
-        # this data overrides what's found in current files. This is done before
-        # the code that merge and coaslesce entries
-        my $fill_blank = __get_fill_blank($fill_blank_data, $f);
-
-        next if $fill_blank->{skip};
-
-        if ( $c =~ /no-info-found/ and $fill_blank->{copyright} ) {
-            $c = $fill_blank->{copyright};
+        else {
+            push @kept_lines, [ $f, $l, $c ];
         }
-        if ($fill_blank->{'override-copyright'}) {
-            _warn "Overriding path $f copyright info\n";
-            $c = $fill_blank->{'override-copyright'};
-        }
-        if ( $l =~/unknown/i and $fill_blank->{license} ) {
-            $l = $fill_blank->{license};
-        }
-        if ($fill_blank->{'override-license'}) {
-            _warn "Overriding path $f license info\n";
-            $l = $fill_blank->{'override-license'};
-        }
-
-        my @no_info_found;
-        if ( $c =~ /no-info-found/ ) {
-            push @no_info_found, 'copyright';
-        }
-        if ( $l =~/unknown/i ) {
-            push @no_info_found, 'license';
-        }
-
-        push @no_info_list , [$f , @no_info_found ] if @no_info_found;
-
-        my $has_info = @no_info_found < 2 ? 1 : 0;
-        $files->{$f} = $copyrights{$c}{$l} //= ($has_info ? $id++ : 0);
-        # say "Storing '$f' : '$c' '$l' has_info: $has_info id ".$files->{$f};
     }
 
     if (@skipped) {
@@ -331,19 +244,170 @@ sub scan_files ( %args ) {
         _warn $msg;
     }
 
-    if (@no_info_list) {
-        my $msg= "The following paths are missing information:\n";
-        map {my ($p,@i) = $_->@*;
-             $msg .= "- $p: missing ".join(' and ', @i)."\n";
-         } @no_info_list;
-        $msg .= "You may want to add a line in debian/fill.copyright.blanks.yml\n\n";
-        _warn $msg;
+    return @kept_lines;
+}
+
+sub __skip_line ($f, $c, $l, $files, $fill_blank_data) {
+    # say "found: $line";
+    return 1 if $files->{$f};       # file already parsed
+
+    # skip copyright because there's no need to have recursive copyrights
+    # skip changelog because it often contains log entries beginning with copyright
+    # and it's a collective work by nature.
+    return 1 if $f =~  m!debian/(copyright|changelog)!;
+
+    # skip copyright infos extracted from license files when they
+    # come from the author of the license and not from the project
+    # author
+    if ($f =~  m/^COPYING|LICENSE$/) {
+        if ($c =~ /Free Software Foundation/) {
+            return 1;
+        }
+        if ($c =~ /Perl Foundation/) {
+            return 1;
+        }
     }
 
-    my @notused = grep { ! $fill_blank_data->{$_}{used} and $_; } sort keys %$fill_blank_data ;
-    if (@notused) {
-        _warn "Warning: the following entries from fill.copyright.blanks.yml were not used\n- '"
-            .join("'\n- '",@notused)."'\n";
+    # skip license files without copyright information (usually plain license)
+    return 1 if $f =~  m/^COPYING|LICENSE$/ && $c =~ /No copyright/;
+
+    # this data overrides what's found in current files. This is done before
+    # the code that merge and coalesce entries
+    my $fill_blank = __get_fill_blank($fill_blank_data, $f);
+
+    return 1 if $fill_blank->{skip};
+    return 0;
+}
+
+my $__extract_rust_info = sub ($file, $c, $l, $current_dir) {
+    my $cargotoml = $current_dir->child($file);
+
+    if ($cargotoml->is_file) {
+        my $toml = $cargotoml->slurp_utf8;
+        my $data = from_toml($toml);
+        my $license = $data->{'package'}{'license'};
+        my $authors = $data->{'package'}{'authors'} // [];
+        if (defined $license) {
+            # Cargo.toml spells AND and OR in capitals
+            # and allows / as a synonym to OR
+            $license =~ s! AND ! and !g;
+            $license =~ s! OR ! or !g;
+            $license =~ s!/! or !g;
+        }
+        return (join("\n ", @$authors) || $c, $license || $l);
+    }
+    return ($c, $l);
+};
+
+my $__extract_nodejs_info = sub ($c_key, $l_key, $file, $c, $l, $current_dir) {
+    my $json_file = $current_dir->child($file);
+
+    if ($json_file->is_file) {
+        my $data = from_json($json_file->slurp_utf8);
+
+        my @c_data;
+        if  (ref $data->{$c_key}) {
+            if (ref $data->{$c_key} eq 'HASH') {
+                if (exists($data->{$c_key}->{name})) {
+                    @c_data = ($data->{$c_key}->{name});
+                }
+            }
+            else {
+                @c_data = ($data->{$c_key}->@*);
+            }
+        } else {
+            @c_data = ($data->{$c_key});
+        }
+
+        my @l_data
+           = ref $data->{$l_key} eq 'ARRAY' ? $data->{$l_key}->@*
+           :                                  $data->{$l_key};
+        return (join("\n ", @c_data) || $c, join(" or ",@l_data) || $l);
+    }
+    return ($c, $l);
+};
+
+my $__extract_json_info = sub ($c_key, $l_key, $file, $c, $l, $current_dir) {
+    my $json_file = $current_dir->child($file);
+
+    if ($json_file->is_file) {
+        my $data = from_json($json_file->slurp_utf8);
+        my @c_data = ref $data->{$c_key} ? $data->{$c_key}->@* : $data->{$c_key};
+        my @l_data = ref $data->{$l_key} ? $data->{$l_key}->@* : $data->{$l_key};
+        return (join("\n ", @c_data) || $c, join(" or ",@l_data) || $l);
+    }
+    return ($c, $l);
+};
+
+my %override = (
+    'Cargo.toml' => $__extract_rust_info,
+    'package.json' => sub {$__extract_nodejs_info->('author','license',@_)},
+    'META.json' => sub {$__extract_json_info->('author','license',@_)},
+    'META6.json' => sub {$__extract_json_info->('authors','license',@_)},
+);
+
+sub __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data) {
+    # override license information by scanning for TOML data
+    if ($override{$f}) {
+        ($c, $l) = $override{$f}->($f, $c, $l, $current_dir);
+        $l =~ s/MIT/Expat/g;
+    }
+
+    $c = __clean_copyright($c);
+
+    $c = __pack_copyright($c);
+
+    # Found in LICENSE files generated by Dist::Zilla
+    $l =~ s/(l?gpl)_(\d)/uc($1)."-$2"/e;
+    $l =~ s/_/./;
+    $l =~ s!GPL-1\+ and/or GPL-1!GPL-1+!;
+
+    my $fill_blank = __get_fill_blank($fill_blank_data, $f);
+    ($c, $l) = __apply_fill_blank($fill_blank, $f, $l, $c);
+    return ($f, $c, $l);
+}
+
+sub __store_line_info($f, $c, $l, $id, $files, $copyrights, $no_info_list) {
+    my @no_info_found;
+    if ( $c =~ /no-info-found/ ) {
+        push @no_info_found, 'copyright';
+    }
+    if ( $l =~/unknown/i ) {
+        push @no_info_found, 'license';
+    }
+
+    push $no_info_list->@* , [$f , @no_info_found ] if @no_info_found;
+
+    my $has_info = @no_info_found < 2 ? 1 : 0;
+    $files->{$f} = $copyrights->{$c}{$l} //= ($has_info ? $id++ : 0);
+    # say "Storing '$f' : '$c' '$l' has_info: $has_info id ".$files->{$f};
+    return $id;
+}
+
+# option to skip UNKNOWN ?
+# load a file to override some entries ?
+sub scan_files ( %args ) {
+    $quiet = $args{quiet} // 0;
+
+    my @line_refs = _get_data_from_files( %args );
+    my $current_dir = $args{from_dir} || path('.');
+
+    my $fill_blank_data = __load_fill_blank_data($current_dir);
+
+    my %copyrights ;
+    my $files = {};
+    my $id = 1;
+    my @no_info_list;
+
+    foreach my $ref (@line_refs) {
+        # say "found: $line";
+        my ($f,$l,$c) = $ref->@*;
+
+        next if __skip_line ($f, $c, $l, $files, $fill_blank_data);
+
+        ($f, $c, $l) = __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data);
+
+        $id = __store_line_info($f, $c, $l, $id, $files, \%copyrights, \@no_info_list) ;
     }
 
     my @copyrights_by_id ;
@@ -354,7 +418,7 @@ sub scan_files ( %args ) {
         }
     }
 
-    warn "No copyright information found" unless keys %$files;
+    _warn_user_about_problems ($files, $fill_blank_data, @no_info_list);
 
     my $merged_c_info = __squash_copyrights_years (\@copyrights_by_id) ;
 
@@ -365,6 +429,27 @@ sub scan_files ( %args ) {
     return ($files, \@copyrights_by_id) ;
 }
 
+sub _warn_user_about_problems ($files, $fill_blank_data, @no_info_list) {
+    if (@no_info_list) {
+        my $msg= "The following paths are missing information:\n";
+        map {my ($p,@i) = $_->@*;
+             $msg .= "- $p: missing ".join(' and ', @i)."\n";
+         } @no_info_list;
+        $msg .= "You may want to add a line in debian/fill.copyright.blanks.yml\n\n";
+        _warn $msg;
+    }
+
+    my @notused = grep { not $fill_blank_data->{$_}{used} and $_; } sort keys %$fill_blank_data ;
+    if (@notused) {
+        _warn "Warning: the following entries from fill.copyright.blanks.yml were not used\n- '"
+            .join("'\n- '",@notused)."'\n";
+    }
+
+    warn "No copyright information found" unless keys %$files;
+
+    return;
+}
+
 sub __to_copyright_structure ($c, $l) {
     return {
         Copyright => $c,
@@ -392,6 +477,29 @@ sub __create_tree_leaf_from_paths ($h,$path,$value) {
     my $last = pop @subpaths;
     map { $h = $h->{$_} ||= {} } @subpaths ;
     $h->{$last} = $value;
+    return;
+}
+
+sub __clean_copyright ($c) {
+    $c =~ s/'//g;
+    $c =~ s/^&copy;\s*//g;
+    $c =~ s/\(c\)\s*//g;
+    $c =~ s/(?<=\b\d{4})\s*-\s*\d{4}(?=\s*-\s*(\d{4})\b)//g;
+    $c =~ s/(\d+)\s*-\s*(\d+)/$1-$2/g;
+    $c =~ s/\b(\d{4}),?\s+([\S^\d])/$1, $2/g;
+    $c =~ s/\s+by\s+//g;
+    $c =~ s/(\\n)*all\s+rights?\s+reserved\.?(\\n)*\s*//gi; # yes there are literal \n
+    $c = 'no-info-found' if $c =~ /^\*No/;
+    $c =~ s/\(r\)//g;
+    $c =~ s!^[\s,/*]|[\s,#/*-]+$!!g;
+    $c =~ s/--/-/g;
+    $c =~ s!\s+\*/\s+! !;
+    # libuv1 has copyright like "2000, -present"
+    $c =~ s![,\s]*-present!'-'.(localtime->year() + 1900)!e;
+    $c =~ s![\x00-\x1f].*!!; # cut off everything after and including the first non-printable
+    # cleanup markdown copyright
+    $c =~ s/\[([\w\s]+)\]\(mailto:([\w@.+-]+)\)/$1 <$2>/;
+    return $c;
 }
 
 sub __pack_copyright ($r) {
@@ -399,7 +507,7 @@ sub __pack_copyright ($r) {
     return $r if $r eq 'no-info-found';
     my %cop;
     $r =~ /^[\s\W]+|[\s\W]+$/g;
-    # split licensescheck output or debfmt data
+    # split licensecheck output or debfmt data
     foreach my $c ( split( m!(?:\s+/\s+)|(?:\s*\n\s*)!, $r) ) {
         my ($owner, @data) = __split_copyright($c);
         return $r unless defined $owner;
@@ -443,7 +551,7 @@ sub __pack_files ($h) {
         $a->[1] cmp $b->[1];
     };
 
-    return sort $sort_path @res ;
+    return (sort $sort_path @res) ;
 }
 
 sub __pack_dir ($h, $pack, @path) {
@@ -462,6 +570,7 @@ sub __pack_dir ($h, $pack, @path) {
     }
 
     push $pack->@*, map { [ $_, $pack_by_id{$_}->@* ];  } keys %pack_by_id ;
+    return;
 }
 
 # find ids that can be merged together
@@ -524,6 +633,7 @@ sub __swap_merged_ids ($files, $merged_c_info) {
             $files->{$name} = "$new_id"  ;
         }
     }
+    return;
 }
 
 sub __coalesce_copyright_years($entries, $owners) {
@@ -556,29 +666,33 @@ sub __coalesce_copyright_years($entries, $owners) {
     return @squashed_c;
 }
 
-# $h is a tree of hash matching the directory structure. Each leaf is a
-# copyright id. Each key is a file name in a directory (not the full path)
-sub __squash ($h, $copyrights_by_id, $p = '') {
-    my %count ;
-
-    # find main license info found in LICENSE or COPYING or README
-    #  file. LICENSE and COPYING are removed from the file list as
-    #  they specify the license author as copyright owner
-    my %main_infos;
+sub __find_main_license_info ($tree_of_ids) {
+    my %main_info;
+    foreach my $name (sort keys %$tree_of_ids) {
+        if ($override{$name}) {
+            my $id = $tree_of_ids->{$name};
+            next unless $id; # skip files without info
+            delete $tree_of_ids->{$name};
+            $main_info{$id} = 1;
+        };
+    }
     foreach my $info_name (qw/README LICENSE LICENCE COPYING/) {
         my $re = qr!^$info_name[.\w]*$!;
-        foreach my $name (sort keys %$h) {
+        foreach my $name (sort keys %$tree_of_ids) {
             if ($name =~ $re) {
-                my $id = $h->{$name};
-                delete $h->{$name} if $info_name ne 'README';
+                my $id = $tree_of_ids->{$name};
+                delete $tree_of_ids->{$name} if $info_name ne 'README';
                 next unless $id; # skip files without info
-                $main_infos{$id} = 1;
+                $main_info{$id} = 1;
             };
         }
     }
+    return \%main_info;
+}
 
+sub __find_main_license_id ($copyrights_by_id, $main_info) {
     my $main_license_id;
-    my @main_info_keys = sort keys %main_infos;
+    my @main_info_keys = sort keys %$main_info;
     if (@main_info_keys == 1) {
         $main_license_id = $main_info_keys[0];
     }
@@ -600,15 +714,19 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             );
         $main_license_id = $#$copyrights_by_id;
     }
+    return $main_license_id;
+}
 
+sub __update_main_license_from_files ($main_license_id, $tree_of_ids, $copyrights_by_id, $path) {
+    my %count ;
     # count the number of times each (c) info is used in this directory.
     # (including the main (c) info of each subdirectory)
-    foreach my $name (sort keys %$h) {
-        my $item = $h->{$name};
+    foreach my $name (sort keys %$tree_of_ids) {
+        my $item = $tree_of_ids->{$name};
         if (ref($item)) {
             # squash may return a plain id, or a hash with '*' => id ,
             # or a non squashable hash
-            $h->{$name} = __squash($item, $copyrights_by_id, $p.'/'.$name);
+            $tree_of_ids->{$name} = __squash_tree_of_copyright_ids($item, $copyrights_by_id, $path.'/'.$name);
         }
         my $id = (ref($item) and defined $item->{'*'}) ? $item->{'*'} : $item ;
 
@@ -631,12 +749,15 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             }
         }
     }
+    return $main_license_id;
+}
 
+sub __prune_files_represented_by_main_license ($main_license_id, $tree_of_ids) {
     # all files associated to the most used (c) info are deleted to
     # be represented by '*' entry
     if (defined $main_license_id) {
-        foreach my $name (sort keys %$h) {
-            my $item = $h->{$name};
+        foreach my $name (sort keys %$tree_of_ids) {
+            my $item = $tree_of_ids->{$name};
             if (ref($item) and defined $item->{'*'} and $item->{'*'} == $main_license_id) {
                 # rename item/* to item/. when covered by ./*
                 # this is a "weak" directory info which is handled specially
@@ -644,15 +765,31 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             }
             if (not ref ($item)) {
                 # delete file that is represented by '*' entry
-                delete $h->{$name} if $item == $main_license_id;
+                delete $tree_of_ids->{$name} if $item == $main_license_id;
             }
         }
 
         # here's the '*' file representing the most used (c) info
-        $h->{'*'} //= $main_license_id;
+        $tree_of_ids->{'*'} //= $main_license_id;
     }
+    return;
+}
 
-    return $h;
+# $tree_of_ids is a tree of hash matching the directory structure. Each leaf is a
+# copyright id. Each key is a file name in a directory (not the full path)
+sub __squash_tree_of_copyright_ids ($tree_of_ids, $copyrights_by_id, $path = '') {
+    # find main license info found in LICENSE or COPYING or README
+    #  file. LICENSE and COPYING are removed from the file list as
+    #  they specify the license author as copyright owner
+    my $main_info = __find_main_license_info($tree_of_ids);
+
+    my $main_license_id = __find_main_license_id($copyrights_by_id, $main_info);
+
+    $main_license_id = __update_main_license_from_files ($main_license_id, $tree_of_ids, $copyrights_by_id, $path);
+
+    __prune_files_represented_by_main_license ($main_license_id, $tree_of_ids);
+
+    return $tree_of_ids;
 }
 
 sub __load_fill_blank_data ($current_dir) {
@@ -664,8 +801,7 @@ sub __load_fill_blank_data ($current_dir) {
 
         warn "Note: loading @fills fixes" if @fills and not $quiet;
         foreach my $file ( @fills) {
-            my $yaml = $file->slurp_utf8;
-            my $data = Load $yaml;
+            my $data = LoadFile($file->stringify);
             foreach my $path (sort keys %$data) {
                 if ($fill_blanks{$path}) {
                     warn "Warning: skipping duplicated fill blank path $path from file $file";
@@ -696,6 +832,23 @@ sub __get_fill_blank ($fbd,$file) {
     return {};
 }
 
+sub __apply_fill_blank($fill_blank, $f, $l, $c) {
+    if ( $c =~ /no-info-found/ and $fill_blank->{copyright} ) {
+        $c = $fill_blank->{copyright};
+    }
+    if ($fill_blank->{'override-copyright'}) {
+        _warn "Overriding path $f copyright info\n";
+        $c = $fill_blank->{'override-copyright'};
+    }
+    if ( $l =~/unknown/i and $fill_blank->{license} ) {
+        $l = $fill_blank->{license};
+    }
+    if ($fill_blank->{'override-license'}) {
+        _warn "Overriding path $f license info\n";
+        $l = $fill_blank->{'override-license'};
+    }
+    return ($c, $l);
+}
 1;
 
 __END__
@@ -738,7 +891,7 @@ I.e files like:
  foo_comp/README: (c) 2018 Joe, GPL-2
  foo_comp/LICENSE: (c) 2017 Max, GPL-3
 
-Yield the following copyright entries:
+yield the following copyright entries:
 
  foo_comp/*:
  Copyright: 2018 Joe
@@ -897,6 +1050,10 @@ Parameters in C<%args>:
 
 set to 1 to suppress progress messages. Should be used only in tests.
 
+=item long
+
+set to 1 to avoid squashing copyright ids. Useful to avoid output with wild cards.
+
 =back
 
 
@@ -904,8 +1061,8 @@ set to 1 to suppress progress messages. Should be used only in tests.
 
 The output of L<licensecheck> is expected to be utf-8. Which means
 that the source files scanned by L<licensecheck> should also be
-encoded in utf-8. In practice, this will impact only copyright owner
-name which may be garbled if comments are not encoded in utf-8.
+encoded in utf-8. This program will abort if invalid utf-8 characters
+are found.
 
 =head1 BUGS
 
@@ -913,14 +1070,14 @@ Extracting license and copyright data from unstructured comments is not reliable
 User must check manually the files when no copyright info is found or when the
 license is unknown.
 
-Source files are assumed to be utf8 (or ascii). Using files with other encoding
-will lead to garbled names of copyright owner. In this case, you can:
+Source files are assumed to be utf8 (or ascii). Using files with invalid characters
+will break C<cme>. In this case, you can:
 
 =over
 
 =item *
 
-Path source files to use utf-8 encoding.
+Patch source files to use utf-8 encoding.
 
 =item *
 
@@ -930,7 +1087,7 @@ encoding.
 
 =item *
 
-File a bug against licensecheck of devscript package to find a better solution.
+File a bug against licensecheck package to find a better solution.
 
 =back
 
