@@ -288,10 +288,20 @@ in ‘/usr/share/common-licenses/GPL-1’.",
         check => {
             'License:ISC text' => qr/Please fill/,
             'License:BSD-2-clause text' => qr/Please fill/,
+            'License:"Artistic-2.0" text' => [
+                qr/The Artistic License 2.0/,
+                {mode => 'custom', value => undef},
+            ],
+            'Files:"3rdparty/libuv/samples/*" License short_name' => "BSD-3-clause and/or Expat",
         },
         wr_check => {
             qq!Files:"3rdparty/dyncall/*" License full_license! => undef,
+            'License:"Artistic-2.0" text' => [
+                qr/The Artistic License 2.0/,
+                {mode => 'custom', value => undef},
+            ],
         },
+        has_not_key => [ License => qr!and/or! ],
         file_contents_unlike => {
             'debian/copyright' => qr/\n\nLicense: ISC\n\n/,
             'debian/copyright' => qr/\n\nLicense: BSD-2-clause\n\n/,
