@@ -86,6 +86,7 @@ my @tests = (
             'package-scripts:t0 prerm' => qr/dummy prerm/,
             'package-scripts:t0/amd64 postrm' => qr/dummy postrm script for amd64/,
             'package-scripts:./amd64 preinst' => qr/dummy preinst script for amd64/,
+            'examples:t0 content:0' => 'examples/'
         },
         log4perl_load_warnings => [[
             User =>  map {(warn => $_)} qr/source Standards-Version/, qr/compat/, (qr/debhelper/) x 2 , qr/Dual dependency/
@@ -238,6 +239,10 @@ my @tests = (
                 qr!Files: debian!,
             ],
         },
+        file_contents_like => {
+            # rules files must finish with a newline
+            'debian/rules' => qr!\n$!,
+        },
 
     },
 
@@ -302,7 +307,11 @@ my @tests = (
             'install:./arm content:0' => '# dummy install for arm',
             'install:batmon.app content:0' => '# dummy batmon.app.install',
             'install:batmon.app/amd64 content:0' => '# dummy batmon.app.install for amd64',
+            'install:batmon.app content:1' => 'debian/batmon.desktop usr/share/applications',
+            'install:batmon.app content:2' => 'usr/plop/batmon.xpm usr/share/pixmaps',
         },
+        apply_fix => 1,
+        file_check_sub => tweak(qw/-compat/),
         load => 'install:"." content:.push("plop.txt usr/share/doc/batmon.app")',
         # this entry is merged in debian/* *if* the Copyright coming from file is normalised
         # Also closes #862368
@@ -344,6 +353,32 @@ my @tests = (
             $tweak_map{config} => qw/joe@foo\.com/,
         }
     },
+
+    {
+        name => 'lintian-overrides',
+        apply_fix => 1,
+        check => {
+            'lintian-overrides:libburn4' => [
+                # check that tag was renamed from shlib-calls-exit
+                qr/exit-in-shared-library/,
+                # check that comment is present
+                qr/decide whether/, qr/busy drives\.\n/,
+            ],
+            'lintian-overrides:.' => [
+                # check that plain overrides file is handled
+                qr/exit-in-shared-library/,
+            ],
+            'source lintian-overrides' => [
+                # check that source overrides file is handled
+                qr/libburn source: source-is-missing/,
+            ],
+        },
+        full_dump => {
+            log4perl_dump_warnings => [
+                [User => warn => qr/dummy-tag/],
+            ],
+        },
+    }
 );
 
 my $cache_file = path('t/model_tests.d/dependency-cache.txt');
